# This file was automatically generated from the `setup.py` script.
version = '4.6.0'
full_version = '4.6.0'
git_revision = '40fc473'
is_released = True

if not is_released:
    version = full_version
